/*
 * Copyright (c) 2006, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.aibonware.inaver.servlet;

import java.io.PrintWriter;

import javax.servlet.http.*;

import com.aibonware.inaver.*;
import com.aibonware.inaver.model.*;
import com.aibonware.inaver.store.*;

public class ChangePasswordContext {
	public final HttpServletRequest request;
	public final HttpServletResponse response;
	public final MasterStore store;
	public final INaver inaver;
	public final PrintWriter out;
	public final UserConfig user;

	public ChangePasswordContext(
		HttpServletRequest request,
		HttpServletResponse response,
		MasterStore store,
		INaver inaver,
		PrintWriter out,
		UserConfig user) {
			this.request = request;
			this.response = response;
			this.store = store;
			this.inaver = inaver;
			this.out = out;
			this.user = user;
	}

	public String createCurrentUrl() {
		return createFullUrl("changepassword");
	}
	public String createFullUrl(String src) {
		String url = inaver.serviceBaseUrl + src;

		if(getLoginMode() > 0) {
			if(url.indexOf('?') < 0) url = url + "?l=" + getLoginMode();
			else url = url + "&l=" + getLoginMode();
		} 

		url = response.encodeURL(url);
		return url;
	}

	public int getLoginMode() {
		String sLoginMode = request.getParameter("l");
		int nLoginMode = tonum(sLoginMode);

		if(nLoginMode <= 0) return 0;
		else return nLoginMode;
	}

	protected int tonum(String s) {
		try {
			return Integer.parseInt(s);
		} catch(Exception e) {
			return -1;
		}
	}
}
