/*
 * Copyright (c) 2006, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.aibonware.inaver.servlet;

import java.io.*;

import javax.servlet.http.*;

import com.aibonware.inaver.*;
import com.aibonware.inaver.model.*;
import com.aibonware.inaver.store.*;
import com.aibonware.inaver.url.*;
import com.aibonware.inaver.task.*;

@SuppressWarnings("serial")
public class TaskServlet extends AuthServlet {
	public TaskServlet() {
		super(AUTO_OPEN_NONE);
	}

	protected AuthServletContext createContext(HttpServletRequest request, HttpServletResponse response, MasterStore store, INaver inaver, PrintWriter out, UserConfig user) throws IOException, StoreException {
		return new AuthServletContext(user, request, response, store, inaver, out, new TaskUrl(request));
	}

	protected void dispContents(AuthServletContext ct) throws StoreException {
		PrintWriter out = ct.out;
		INaver inaver = ct.inaver;
		HttpServletRequest request = ct.request;
		HttpServletResponse response = ct.response;
		UserConfig user = null;
		HttpSession session = request.getSession(false);
		if(session != null) user = (UserConfig)session.getAttribute("userconfig");
		TaskUrl taskUrl = (TaskUrl)ct.authUrl;
		int nLoginMode = taskUrl.loginMode;

		if(user == null) user = new UserConfig();

		int taskId = taskUrl.taskId;
		
        Log.info(user.userName + " query task " + taskId);

		if(nLoginMode == 0) {
			out.print("۸޲݂Ă܂");
		} else {
			UserTask task = inaver.getUserTask(user, taskId);
	
			if(task == null) {
				out.print("invalid task id " + taskId);
			} else if(taskUrl.cancel == TaskUrl.CANCEL_EXECUTE) {
				task.cancel();
				out.print("LZ܂");
				INaver.getInstance().removeUserTask(task);
			} else if(task.exception != null) {
				task.exception.printStackTrace(out);
				INaver.getInstance().removeUserTask(task);
			} else if(!task.completed) {
				out.print("task not complete");
			} else {
				task.dispResult(user, out ,request ,response);
				INaver.getInstance().removeUserTask(task);
			}
		}
	}
}
